package com.eco.plugin.wink.workflow.controller;

import com.eco.plugin.wink.workflow.db.bean.DBLinkEntity;
import com.eco.plugin.wink.workflow.db.bean.FormDefEntity;
import com.eco.plugin.wink.workflow.db.bean.FormDetailEntity;
import com.eco.plugin.wink.workflow.db.controller.WorkFlowController;
import com.eco.plugin.wink.workflow.utils.DateUtilSelf;
import com.eco.plugin.wink.workflow.utils.ResponseUtils;
import com.eco.plugin.wink.workflow.utils.Utils;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.PostMapping;
import com.fr.third.springframework.web.bind.annotation.RequestBody;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Controller
@LoginStatusChecker(required = false)
@FunctionRecorder
public class ControllerSelf {

    /**
     * 获取全部数据库链接
     * @param req
     * @param res
     */
    @PostMapping(value = "/workflow/getAllDbLink")
    @ResponseBody
    public void getAllDbLink(HttpServletRequest req,HttpServletResponse res){
        List<DBLinkEntity> dbLinkList = WorkFlowController.getAllDBLink();
        ResponseUtils.response(res,new JSONObject().put("data",dbLinkList));
    }

    /**
     * 根据id获取单个数据库链接
     * @param req
     * @param res
     */
    @PostMapping(value = "/workflow/getDbLinkById")
    @ResponseBody
    public DBLinkEntity getDbLinkById(HttpServletRequest req,HttpServletResponse res){
        String id = Utils.getRequestBody(req).getString("id");
        DBLinkEntity dbLink = WorkFlowController.getDBLinkById(id);
        return dbLink;
    }

    @PostMapping(value = "/workflow/addDbLink")
    @ResponseBody
    public void addDbLink(HttpServletRequest req, HttpServletResponse res, @RequestBody DBLinkEntity dbLink){
        String now = DateUtilSelf.getNow();
        if(Utils.isNullStr(dbLink.getId())){
            dbLink.setId(UUID.randomUUID().toString());
            dbLink.setCreatetime(now);
        }

        if(Utils.isNotNullStr(dbLink.getId())){
            dbLink.setCreatetime(WorkFlowController.getDBLinkById(dbLink.getId()).getCreatetime());
        }

        dbLink.setUpdatetime(now);
        WorkFlowController.singleDBLink(dbLink,null);
        ResponseUtils.successResponse(res,"success");
    }

    @PostMapping(value = "/workflow/deleteDBLink")
    @ResponseBody
    public void deleteDBLink(HttpServletRequest req, HttpServletResponse res, @RequestBody DBLinkEntity dbLink){
        WorkFlowController.singleDBLink(null,dbLink);
        ResponseUtils.successResponse(res,"success");
    }

    @PostMapping(value = "/workflow/importForm")
    @ResponseBody
    public void importForm(HttpServletRequest req, HttpServletResponse res){
        JSONObject json = Utils.getRequestBody(req);
        String paramstr = json.getString("data");
        JSONArray params = new JSONArray(paramstr);
        for(int i=0;i<params.length();i++){
            JSONObject param = params.getJSONObject(i);
            String formid = addForm(param);

            addFormField(formid,param.getJsonArray("fields"));
        }


        ResponseUtils.successResponse(res,"success");
    }

    @PostMapping(value = "/workflow/getFormList")
    @ResponseBody
    public void getFormList(HttpServletRequest req, HttpServletResponse res){
        JSONObject json = Utils.getRequestBody(req);

        String formid = addForm(json);

        addFormField(formid,json.getJsonArray("fields"));
        ResponseUtils.successResponse(res,"success");
    }

    private void addFormField(String formid, JSONArray fields) {
        List<FormDetailEntity> formDetailEntitys = new ArrayList<>();

        for(int i=0;i< fields.length();i++){
            FormDetailEntity fde = new FormDetailEntity();
            String id = UUID.randomUUID().toString();
            JSONObject field = fields.getJSONObject(i);
            String fieldname = field.getString("fieldname");
            String fielddesc = field.getString("fielddesc");
            fde.setId(id);
            fde.setFormid(formid);
            fde.setFieldname(fieldname);
            fde.setFielddec(fielddesc);
            fde.setSort(String.valueOf(i));
            formDetailEntitys.add(fde);
        }
        
        WorkFlowController.batch(formDetailEntitys,null);
    }

    /**
     * 添加表单
     * @param json
     * @return
     */
    private String addForm(JSONObject json) {
        String tablename = json.getString("tablename");
        String formname = json.getString("formname");
        FormDefEntity fde = new FormDefEntity();
        String id = UUID.randomUUID().toString();
        fde.setId(id);
        fde.setTablename(tablename);
        fde.setFormname(formname);
        String now = DateUtilSelf.getNow();
        fde.setCreatetime(now);
        fde.setUpdatetime(now);
        //可用
        fde.setStatus("0");
        WorkFlowController.singleForm(fde,null);

        return id;
    }
}
