package com.eco.plugin.wink.workflow.controller;

import com.eco.plugin.wink.workflow.db.bean.FormDefEntity;
import com.eco.plugin.wink.workflow.db.bean.FormDetailEntity;
import com.eco.plugin.wink.workflow.db.bean.WorkflowDefEntity;
import com.eco.plugin.wink.workflow.db.bean.WorkflowNodeEntity;
import com.eco.plugin.wink.workflow.db.controller.FormDbController;
import com.eco.plugin.wink.workflow.db.controller.WorkflowConfigDbController;
import com.eco.plugin.wink.workflow.utils.*;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.PostMapping;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;
import org.nfunk.jep.function.Str;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

@Controller
@LoginStatusChecker(required = false)
public class DataProcessingController {

    @PostMapping(value = "/workflow/dataprocess/add")
    @ResponseBody
    public void add(HttpServletRequest req, HttpServletResponse res) throws Exception {
        JSONObject json = Utils.getRequestBody(req);
        String formid = json.getString("formid");
        FormDefEntity form = FormDbController.getById(formid);

        JSONArray fields = json.getJSONArray("field");

        String sql = generateInsertSql(req,form,fields);
        FRUtils.FRLogInfo("sql:"+sql);
        JDBCUtils.update(sql);
    }

    @PostMapping(value = "/workflow/dataprocess/getPageData")
    @ResponseBody
    public void getPageData(HttpServletRequest req, HttpServletResponse res) throws Exception {
        JSONObject json = Utils.getRequestBody(req);
        String formid = json.getString("formid");
        int page = json.getInt("page");
        int count = json.getInt("count");
        String createTime = json.getString("createtime");

        //1、数据录入 2、数据审核
        String from = json.getString("from");

        FormDefEntity form = FormDbController.getById(formid);

        String sql = generateQuerySql(form);

        if(Utils.isNotNullStr(createTime)){
            sql += " where createtime >= "+createTime;
        }

        String username =FRUserUtils.getCurrentUser(req).getUserName();

        if("1".equals(from)){
            sql+= sql.contains("where") ? " and createuser = '"+username+"'": " where createuser='"+username+"'";
        }else{
            sql+= sql.contains("where") ? " and currentuser like '%["+username+"]%'" : " where currentuser like '%["+username+"]%'";
        }

        sql += " order by createtime limit "+(page - 1)*count+","+count;

        FRUtils.FRLogInfo("sql:"+sql);

        String[] column = getColumn(form);

        List<Map<String,Object>> items = JDBCUtils.getResult(sql,column,formid);

        String countSql = "select count(id) from " + form.getTablename();

        Map<String,Object> result = new HashMap<>();
        result.put("page",page);
        result.put("count",count);
        result.put("items",items);
        ResponseUtils.successResponse(res,new JSONObject().put("page",page).put("count",count).put("items",items).put("total",JDBCUtils.getCount(countSql)).toString());
    }

    @PostMapping(value = "/workflow/dataprocess/getData")
    @ResponseBody
    public Map<String,Object> getData(HttpServletRequest req, HttpServletResponse res) throws Exception {
        JSONObject json = Utils.getRequestBody(req);
        String formid = json.getString("formid");
        String dataid = json.getString("dataid");

        FormDefEntity form = FormDbController.getById(formid);
        String sql = generateQuerySql(form);

        sql += " where id='"+dataid+"'";
        String[] column = getColumn(form);

        List<Map<String,Object>> items = JDBCUtils.getResult(sql,column,formid);
        return items.get(0);
    }

    @PostMapping(value = "/workflow/dataprocess/delete")
    @ResponseBody
    public void delete(HttpServletRequest req, HttpServletResponse res) throws Exception {
        JSONObject json = Utils.getRequestBody(req);
        String formid = json.getString("formid");
        String dataid = json.getString("dataid");

        FormDefEntity form = FormDbController.getById(formid);
        String sql = generateDeleteSql(form);

        sql += " where id='"+dataid+"'";

        JDBCUtils.update(sql);
    }

    private String generateDeleteSql(FormDefEntity form) {
        String sql = "delete from "+form.getTablename();
        return sql ;
    }

    /**
     * 获取列名
     * @param form
     * @return
     */
    private String[] getColumn(FormDefEntity form){
        List<FormDetailEntity> fields = FormDbController.getFieldByFormId(form.getId());
        String[] column = new String[fields.size()];

        for(int i=0;i<fields.size();i++){
            column[i] = fields.get(i).getFieldname();
        }

        return column;
    }

    /**
     * 生成查询sql
     * @param form
     * @return
     */
    private String generateQuerySql(FormDefEntity form) {
        String sql = "select ";
        List<FormDetailEntity> fields = FormDbController.getFieldByFormId(form.getId());

        for(FormDetailEntity field : fields){
            sql += field.getFieldname()+",";
        }

        return sql.substring(0,sql.length()-1) + " from " + form.getTablename();
    }

    /**
     * 生成插入sql
     * @param form
     * @param fields
     * @return
     */
    private static String generateInsertSql(HttpServletRequest req,FormDefEntity form, JSONArray fields) throws Exception {
        String insertsql = "insert into " + form.getTablename() +"(id";
        String valuesql =" values('"+UUID.randomUUID().toString()+"'";


        for(int i=0;i<fields.length();i++){
            JSONObject field = fields.getJSONObject(i);
            insertsql+= ","+field.getString("field");
            valuesql+=",'"+field.getString("value")+"'";
        }

        insertsql+=",status,createtime,createuser)";

        valuesql+= ",'','"+DateUtilSelf.getNow()+"','"+FRUserUtils.getCurrentUser(req).getUserName()+"')";

        return insertsql+valuesql;
    }

}
