package com.eco.plugin.wink.workflow.controller;

import com.eco.plugin.wink.workflow.db.bean.FormDefEntity;
import com.eco.plugin.wink.workflow.db.bean.WorkflowDefEntity;
import com.eco.plugin.wink.workflow.db.bean.WorkflowNodeEntity;
import com.eco.plugin.wink.workflow.db.controller.FormDbController;
import com.eco.plugin.wink.workflow.db.controller.WorkflowConfigDbController;
import com.eco.plugin.wink.workflow.utils.DateUtilSelf;
import com.eco.plugin.wink.workflow.utils.ResponseUtils;
import com.eco.plugin.wink.workflow.utils.Utils;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.PostMapping;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Controller
@LoginStatusChecker(required = false)
public class WorkflowConfigController {

    @PostMapping(value = "/workflow/config/importWorkFlow")
    @ResponseBody
    public void importWorkFlow(HttpServletRequest req, HttpServletResponse res){
        JSONObject json = Utils.getRequestBody(req);
        String paramstr = json.getString("data");
        JSONArray params = new JSONArray(paramstr);
        for(int i=0;i<params.length();i++){
            String result = importFlow(params.getJSONObject(i));
        }


        ResponseUtils.successResponse(res,"success");
    }

    /**
     * 导入流程
     * @param param
     * @return
     */
    private String importFlow(JSONObject param) {
        String id = addWorkflowDef(param);
        String result = addWorkflowNode(id,param.getJsonArray("workflowNodes"));
        return result;
    }

    /**
     * 添加流程节点
     * @param id
     * @param workflowNodes
     * @return
     */
    private String addWorkflowNode(String id, JSONArray workflowNodes) {
        String[] ids = id.split(",");
        String formid = ids[1];
        String workflowDefId = ids[0];
        List<WorkflowNodeEntity> nodelist = new ArrayList<>();
        for(int i=0;i<workflowNodes.size();i++){
            JSONObject node = workflowNodes.getJSONObject(i);
            WorkflowNodeEntity workflowNodeEntity = new WorkflowNodeEntity();
            workflowNodeEntity.setId(UUID.randomUUID().toString());
            workflowNodeEntity.setWorkflowid(workflowDefId);
            workflowNodeEntity.setFormid(formid);
            workflowNodeEntity.setNode(node.getString("node"));
            workflowNodeEntity.setNodename(node.getString("nodename"));
            workflowNodeEntity.setSort(String.valueOf(i));
            workflowNodeEntity.setRolename(node.getString("rolename"));
            nodelist.add(workflowNodeEntity);
        }

        WorkflowConfigDbController.batchWorkflowNode(nodelist,null);

        return "";
    }

    /**
     * 添加流程定义
     * @param param
     * @return
     */
    private String addWorkflowDef(JSONObject param){
        String tablename = param.getString("tablename");
        FormDefEntity formDefEntity = FormDbController.getByTablename(tablename);
        String formid = formDefEntity.getId();
        String workflowcode = param.getString("workflowcode");
        String workflowname = param.getString("workflowname");
        String createtime = DateUtilSelf.getNow();

        WorkflowDefEntity workflowDefEntity = new WorkflowDefEntity();
        String id = UUID.randomUUID().toString();
        workflowDefEntity.setId(id);
        workflowDefEntity.setWorkflowcode(workflowcode);
        workflowDefEntity.setWorkflowname(workflowname);
        workflowDefEntity.setCreatetime(createtime);
        workflowDefEntity.setFormid(formid);

        WorkflowConfigDbController.singleWorkflowDef(workflowDefEntity,null);

        return id+","+formid;
    }

    /**
     * 获取全部流程定义
     * @param req
     * @param res
     */
    @PostMapping(value = "/workflow/config/getWorkflowDefList")
    @ResponseBody
    public void getFormList(HttpServletRequest req,HttpServletResponse res){
        List<WorkflowDefEntity> workflowDefEntityList = WorkflowConfigDbController.getWrokflowDefList();
        ResponseUtils.response(res,new JSONObject().put("data",workflowDefEntityList));
    }

    /**
     * 根据流程id获取流程节点
     * @param req
     * @param res
     */
    @PostMapping(value = "/workflow/config/getNodeByWorkflowId")
    @ResponseBody
    public void getFieldByFormId(HttpServletRequest req,HttpServletResponse res){
        JSONObject param = Utils.getRequestBody(req);
        String workflowid = param.getString("id");
        List<WorkflowNodeEntity> workflowNodeEntityList = WorkflowConfigDbController.getNodesByWorkflowId(workflowid);
        ResponseUtils.response(res,new JSONObject().put("data",workflowNodeEntityList));
    }
}
