package com.eco.plugin.wink.workflow.controller;

import com.eco.plugin.wink.workflow.db.bean.FormDefEntity;
import com.eco.plugin.wink.workflow.db.bean.WorkflowDefEntity;
import com.eco.plugin.wink.workflow.db.bean.WorkflowNodeEntity;
import com.eco.plugin.wink.workflow.db.bean.WorkflowProcessEntity;
import com.eco.plugin.wink.workflow.db.controller.FormDbController;
import com.eco.plugin.wink.workflow.db.controller.WorkFlowProcessController;
import com.eco.plugin.wink.workflow.db.controller.WorkflowConfigDbController;
import com.eco.plugin.wink.workflow.kit.UserServiceKit;
import com.eco.plugin.wink.workflow.utils.*;
import com.fr.decision.authority.data.User;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.stable.query.data.DataList;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.PostMapping;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

@Controller
@LoginStatusChecker(required = false)
public class WorkflowProcessController {

    /**
     * 发起流程
     * @param req
     * @param res
     */
    @PostMapping(value = "/workflow/process/start")
    @ResponseBody
    public void importWorkFlow(HttpServletRequest req, HttpServletResponse res){
        JSONObject json = Utils.getRequestBody(req);

        //处理当前流程节点
        try {
            operateWorkflow(json,req,"start");
        } catch (Exception e) {
            ResponseUtils.failedResponse(res,"发起失败:"+e.getMessage());
            return ;
        }

        ResponseUtils.successResponse(res,"发起成功");
    }

    /**
     * 审核流程
     * @param req
     * @param res
     */
    @PostMapping(value = "/workflow/process/approve")
    @ResponseBody
    public void approve(HttpServletRequest req, HttpServletResponse res){
        JSONObject json = Utils.getRequestBody(req);

        //处理当前流程节点
        try {
            operateWorkflow(json,req,"approve");
        } catch (Exception e) {
            ResponseUtils.failedResponse(res,"审核失败:"+e.getMessage());
            return ;
        }

        ResponseUtils.successResponse(res,"审核成功");
    }

    /**
     * 处理流程
     * @param param
     * @param req
     * @param type start 发起 approve审核
     */
    private void operateWorkflow(JSONObject param,HttpServletRequest req,String type) throws Exception {
        //参数
        String formid = param.getString("formid");
        String dataid = param.getString("dataid");
        String comment = param.getString("comment");
        //表单定义
        FormDefEntity form = FormDbController.getById(formid);
        //流程定义
        WorkflowDefEntity workflowDefEntity = WorkflowConfigDbController.getByFormid(formid);
        //流程节点
        List<WorkflowNodeEntity> nodes = WorkflowConfigDbController.getNodesByWorkflowId(workflowDefEntity.getId());
        //当前用户
        User user = FRUserUtils.getCurrentUser(req);

        //流程执行列表
        List<WorkflowProcessEntity> processList = new ArrayList<>();

        if(!"start".equals(type)){
            processList = WorkFlowProcessController.getByFormidAndDataid(formid,dataid);
        }

        List<WorkflowProcessEntity> process = new ArrayList<>();
        WorkflowProcessEntity currentNodeProcess = getCurrenNode(user,formid,dataid,nodes,processList);
        WorkflowProcessEntity nextNodeProcess = getNextNodeProcess(currentNodeProcess,nodes,formid,dataid);

        if(Utils.isNotNullStr(comment)){
            currentNodeProcess.setComment(comment);
        }

        process.add(currentNodeProcess);
        process.add(nextNodeProcess);
        //操作流程处理表
        WorkFlowProcessController.batch(process,null);
        updateDataStatus(dataid,formid,nextNodeProcess,nodes);
    }

    /**
     * 获取下个节点
     * @param currentNodeProcess
     * @param nodes
     * @param formid
     * @param dataid
     * @return
     * @throws Exception
     */
    private WorkflowProcessEntity getNextNodeProcess(WorkflowProcessEntity currentNodeProcess, List<WorkflowNodeEntity> nodes, String formid, String dataid) throws Exception {
        WorkflowProcessEntity current = new WorkflowProcessEntity();
        current.setId(UUID.randomUUID().toString());
        current.setFormid(formid);
        current.setDataid(dataid);
        current.setCreatetime(DateUtilSelf.getNow());
        current.setLastnode(currentNodeProcess.getCurrentnode());
        //结束
        if(Utils.isNullStr(currentNodeProcess.getNextnode())){
            current.setCurrentnode("结束");
            WorkflowNodeEntity finnalNode = nodes.get(nodes.size() -1 );
            String sort = String.valueOf(Integer.valueOf(finnalNode.getSort()) + 1);
            current.setSort(sort);
        }else{
            Map<String,WorkflowNodeEntity> map = getNextAndThirdNodeByNode(currentNodeProcess.getNextnode(),nodes);
            WorkflowNodeEntity currentNode = map.get("current");
            WorkflowNodeEntity nextNode = map.get("next");

            current.setCurrentnode(currentNode.getNode());
            String checker = getChecker(currentNode.getRolename());
            current.setOperateruser(checker);

            if(nextNode != null){
                current.setNextnode(nextNode.getNode());
            }
            current.setSort(currentNode.getSort());
        }

        return current;
    }

    /**
     * 获取当前节点
     * @param user
     * @param formid
     * @param dataid
     * @param nodes
     * @param processList
     * @return
     */
    private WorkflowProcessEntity getCurrenNode(User user, String formid, String dataid, List<WorkflowNodeEntity> nodes,List<WorkflowProcessEntity> processList){
        boolean isStart = processList == null || processList.size() <= 0;
        String currentNodeCode = isStart ? "" : processList.get(processList.size() -1).getCurrentnode();
        Map<String,WorkflowNodeEntity> map = getCurrentAndNextByNode(currentNodeCode,nodes);
        WorkflowNodeEntity currentNode = map.get("current");
        WorkflowNodeEntity nextNode = map.get("next");

        WorkflowProcessEntity current = new WorkflowProcessEntity();

        //发起
        if(isStart){
            current.setId(UUID.randomUUID().toString());
            current.setFormid(formid);
            current.setDataid(dataid);
            current.setCurrentnode(currentNode.getNode());
            current.setNextnode(nextNode.getNode());
            current.setOperateruser(user.getUserName());
            current.setSort(currentNode.getSort());
            current.setCreatetime(DateUtilSelf.getNow());
        }else{
            current = processList.get(processList.size() -1);
            current.setOperateruser(current.getOperateruser()+"("+user.getUserName()+")");
        }

        return current;
    }



    /**
     * 修改数据审核状态
     * @param
     */
    private void updateDataStatus(String dataid,String formid,WorkflowProcessEntity current,List<WorkflowNodeEntity> nodes) throws Exception {
        FormDefEntity formDef = FormDbController.getById(formid);
        String tablename = formDef.getTablename();
        String nodename = "";

        String currentuser = "";
        if(current.getCurrentnode().equals("结束")){
            nodename = "结束";
        }else{
            for (WorkflowNodeEntity node : nodes){
                if(node.getNode().equals(current.getCurrentnode())){
                    nodename = node.getNodename();
                    currentuser = getCheckerForData(node.getRolename());
                    break;
                }
            }
        }

        String sql = "update " + tablename + " set status='"+nodename+"',currentuser='"+currentuser+"' where id = '"+dataid+"'";

        JDBCUtils.update(sql);
    }

    /**
     * 根据节点编码获取当前节点和下个节点
     * @param node
     * @param nodes
     * @return
     */
    private Map<String,WorkflowNodeEntity> getCurrentAndNextByNode(String node,List<WorkflowNodeEntity> nodes){
        Map<String,WorkflowNodeEntity> map = new HashMap<>();

        //发起流程 直接返回前两个节点
        if(Utils.isNullStr(node)){
            map.put("current",nodes.get(0));
            map.put("next",nodes.get(1));
            return map;
        }

        //审核流程
        for(int i=0;i<nodes.size();i++){
            WorkflowNodeEntity currentNode = nodes.get(i);
            if(!currentNode.getNode().equals(node)){
                continue;
            }

            map.put("current",currentNode);
            map.put("next",null);

            if(i != nodes.size() -1){
                map.put("next",nodes.get(nodes.size() -1));
            }

            break;
        }

        return map;
    }

    /**
     * 根据当前节点获取下个和下下个节点
     * @param node
     * @param nodes
     * @return
     */
    private Map<String,WorkflowNodeEntity> getNextAndThirdNodeByNode(String node,List<WorkflowNodeEntity> nodes){
        Map<String,WorkflowNodeEntity> map = new HashMap<>();

        for(int i=0;i<nodes.size();i++){
            WorkflowNodeEntity currentNode = nodes.get(i);
            if(!currentNode.getNode().equals(node)){
                continue;
            }

            map.put("current",currentNode);
            map.put("next",null);

            if(i != nodes.size() -1){
                map.put("next",nodes.get(nodes.size() -1));
            }

            break;
        }

        return map;
    }

    /**
     * 根据角色名称获取审核人
     * @param rolename
     * @return
     */
    private String getChecker(String rolename) throws Exception {
//        return "xiamaofa";
        List<User> users = UserServiceKit.getInstance().getAllUsersByRolename(rolename);
        String checker = "";
        for(User user : users){
            checker+=","+user.getUserName();
        }

        checker = checker.replaceFirst(",","");
        return checker;
    }

    /**
     * 根据角色名称获取审核人-修改data审核人用
     * @param rolename
     * @return
     */
    private String getCheckerForData(String rolename) throws Exception {
//        return "[xiamaofa]";
        List<User> users = UserServiceKit.getInstance().getAllUsersByRolename(rolename);
        String checker = "";
        for(User user : users){
            checker+=",["+user.getUserName()+"]";
        }

        checker = checker.replaceFirst(",","");
        return checker;
    }
}
