package com.eco.plugin.wink.workflow.db.controller;

import com.eco.plugin.wink.workflow.db.bean.FormDefEntity;
import com.eco.plugin.wink.workflow.db.bean.FormDetailEntity;
import com.eco.plugin.wink.workflow.db.dao.FormDefDao;
import com.eco.plugin.wink.workflow.db.dao.FormFieldDao;
import com.eco.plugin.wink.workflow.utils.FRUtils;
import com.fr.db.fun.impl.AbstractDBAccessProvider;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.db.dao.DAOProvider;

import java.util.ArrayList;
import java.util.List;

/**
 * @author wink
 * @version 10.0
 * Created by wink on 2021-07-29
 **/
@EnableMetrics
public class FormDbController extends AbstractDBAccessProvider {

    private static DBAccessor accessor;

    public static DBAccessor getAccessor() {
        return accessor;
    }

    @Override
    public DAOProvider[] registerDAO() {
        return new DAOProvider[]{
                FormDefDao.DAO, FormFieldDao.DAO
        };
    }

    @Override
    public void onDBAvailable(DBAccessor accessor) {
        FormDbController.accessor = accessor;
    }

    /**
     * 单个操作表单
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean singleForm(FormDefEntity addOrUpdate, FormDefEntity delete){
        //新增或者删除
        if(addOrUpdate != null ){
            try{
                accessor.runDMLAction(new DBAction<Boolean>() {
                    @Override
                    public Boolean run(DAOContext context) throws Exception {
                        FormDefEntity ae =context.getDAO(FormDefDao.class).getById(addOrUpdate.getId());
                        if(ae != null ){
                            context.getDAO(FormDefDao.class).update(addOrUpdate);
                        }else{
                            context.getDAO(FormDefDao.class).add(addOrUpdate);
                        }
                        return true;
                    }
                });
            }catch(Throwable e){
                FRUtils.FRLogError("singleForm exception ->"+e.getMessage() + addOrUpdate.toString());
            }
        }

        if(delete != null ){
            try{
                accessor.runDMLAction(new DBAction<Boolean>() {
                    @Override
                    public Boolean run(DAOContext context) throws Exception {
                        context.getDAO(FormDefDao.class).remove(delete.getId());
                        return true;
                    }
                });
            }catch(Throwable e){
                FRUtils.FRLogError("singleForm delete exception ->"+e.getMessage() + delete.toString());
            }
        }

        return true;
    }

    /**
     * 批量操作表单字段
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean batch(List<FormDetailEntity> addOrUpdate, List<FormDetailEntity> delete){
        //新增或者删除
        if(addOrUpdate != null && addOrUpdate.size() > 0){
            for(final FormDetailEntity dbe : addOrUpdate){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            FormDetailEntity ae =context.getDAO(FormFieldDao.class).getById(dbe.getId());
                            if(ae != null ){
                                context.getDAO(FormFieldDao.class).update(dbe);
                            }else{
                                context.getDAO(FormFieldDao.class).add(dbe);
                            }
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch addOrUpdate formfield exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        if(delete != null && delete.size() > 0){
            for(final FormDetailEntity dbe : delete){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            context.getDAO(FormFieldDao.class).remove(dbe.getId());
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch  formfiled delete exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        return true;
    }

    /**
     * 获取所有表单信息
     * @param
     * @return
     */
    public static List<FormDefEntity> getFormList(){
        try{
            return accessor.runQueryAction(new DBAction<List<FormDefEntity>>() {
                @Override
                public List<FormDefEntity> run(DAOContext context) throws Exception {
                    List<FormDefEntity> result = context.getDAO(FormDefDao.class).getAll();

                    return result == null ? new ArrayList<FormDefEntity>() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getAllDBLink:"+e.getMessage());
        }

        return new ArrayList<FormDefEntity>();
    }

    /**
     * 根据tablename获取表单
     * @param
     * @return
     */
    public static FormDefEntity getByTablename(final String tablename){
        try{
            return accessor.runQueryAction(new DBAction<FormDefEntity>() {
                @Override
                public FormDefEntity run(DAOContext context) throws Exception {
                    FormDefEntity result = context.getDAO(FormDefDao.class).getByTablename(tablename);

                    return result == null ? new FormDefEntity() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getAllDBLink:"+e.getMessage());
        }

        return new FormDefEntity();
    }

    /**
     * 根据id获取表单
     * @param
     * @return
     */
    public static FormDefEntity getById(final String id){
        try{
            return accessor.runQueryAction(new DBAction<FormDefEntity>() {
                @Override
                public FormDefEntity run(DAOContext context) throws Exception {
                    FormDefEntity result = context.getDAO(FormDefDao.class).getById(id);

                    return result == null ? new FormDefEntity() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getById:"+e.getMessage());
        }

        return new FormDefEntity();
    }

    /**
     * 根据表单id获取字段列表
     * @param
     * @return
     */
    public static List<FormDetailEntity> getFieldByFormId(String formid){
        try{
            return accessor.runQueryAction(new DBAction<List<FormDetailEntity>>() {
                @Override
                public List<FormDetailEntity> run(DAOContext context) throws Exception {
                    List<FormDetailEntity> result = context.getDAO(FormFieldDao.class).getFieldById(formid);

                    return result == null ? new ArrayList<FormDetailEntity>() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getAllDBLink:"+e.getMessage());
        }

        return new ArrayList<FormDetailEntity>();
    }
}
