package com.eco.plugin.wink.workflow.db.controller;

import com.eco.plugin.wink.workflow.db.bean.DBLinkEntity;
import com.eco.plugin.wink.workflow.db.bean.FormDefEntity;
import com.eco.plugin.wink.workflow.db.bean.FormDetailEntity;
import com.eco.plugin.wink.workflow.db.dao.DBLinkDao;
import com.eco.plugin.wink.workflow.db.dao.FormDefDao;
import com.eco.plugin.wink.workflow.db.dao.FormFieldDao;
import com.eco.plugin.wink.workflow.utils.FRUtils;
import com.fr.db.fun.impl.AbstractDBAccessProvider;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.db.dao.DAOProvider;

import java.util.ArrayList;
import java.util.List;

/**
 * @author wink
 * @version 10.0
 * Created by wink on 2021-07-29
 **/
@EnableMetrics
public class WorkFlowController extends AbstractDBAccessProvider {

    private static DBAccessor accessor;

    public static DBAccessor getAccessor() {
        return accessor;
    }

    @Override
    public DAOProvider[] registerDAO() {
        return new DAOProvider[]{
                DBLinkDao.DAO, FormDefDao.DAO, FormFieldDao.DAO
        };
    }

    @Override
    public void onDBAvailable(DBAccessor accessor) {
        WorkFlowController.accessor = accessor;
    }
    
    /**
     * 单个操作数据库链接
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean singleDBLink(DBLinkEntity addOrUpdate,DBLinkEntity delete){
        //新增或者删除
        if(addOrUpdate != null ){
            try{
                accessor.runDMLAction(new DBAction<Boolean>() {
                    @Override
                    public Boolean run(DAOContext context) throws Exception {
                        DBLinkEntity ae =context.getDAO(DBLinkDao.class).getById(addOrUpdate.getId());
                        if(ae != null ){
                            context.getDAO(DBLinkDao.class).update(addOrUpdate);
                        }else{
                            context.getDAO(DBLinkDao.class).add(addOrUpdate);
                        }
                        return true;
                    }
                });
            }catch(Throwable e){
                FRUtils.FRLogError("singleDBLink exception ->"+e.getMessage() + addOrUpdate.toString());
            }
        }

        if(delete != null ){
            try{
                accessor.runDMLAction(new DBAction<Boolean>() {
                    @Override
                    public Boolean run(DAOContext context) throws Exception {
                        context.getDAO(DBLinkDao.class).remove(delete.getId());
                        return true;
                    }
                });
            }catch(Throwable e){
                FRUtils.FRLogError("singleDBLink delete exception ->"+e.getMessage() + delete.toString());
            }
        }

        return true;
    }

    /**
     * 获取所有数据库链接信息
     * @param
     * @return
     */
    public static List<DBLinkEntity> getAllDBLink(){
        try{
            return accessor.runQueryAction(new DBAction<List<DBLinkEntity>>() {
                @Override
                public List<DBLinkEntity> run(DAOContext context) throws Exception {
                    List<DBLinkEntity> result = context.getDAO(DBLinkDao.class).getAll();

                    return result == null ? new ArrayList<DBLinkEntity>() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getAllDBLink:"+e.getMessage());
        }

        return new ArrayList<DBLinkEntity>();
    }

    /**
     * 根据id获取信息
     * @param id
     * @return
     */
    public static DBLinkEntity getDBLinkById(final String id){
        try{
            return accessor.runQueryAction(new DBAction<DBLinkEntity>() {
                @Override
                public DBLinkEntity run(DAOContext context) throws Exception {
                    DBLinkEntity result = context.getDAO(DBLinkDao.class).getById(id);

                    return result == null ? new DBLinkEntity() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getDBLinkById:"+e.getMessage());
        }

        return new DBLinkEntity();
    }

    /**
     * 单个操作表单
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean singleForm(FormDefEntity addOrUpdate, FormDefEntity delete){
        //新增或者删除
        if(addOrUpdate != null ){
            try{
                accessor.runDMLAction(new DBAction<Boolean>() {
                    @Override
                    public Boolean run(DAOContext context) throws Exception {
                        FormDefEntity ae =context.getDAO(FormDefDao.class).getById(addOrUpdate.getId());
                        if(ae != null ){
                            context.getDAO(FormDefDao.class).update(addOrUpdate);
                        }else{
                            context.getDAO(FormDefDao.class).add(addOrUpdate);
                        }
                        return true;
                    }
                });
            }catch(Throwable e){
                FRUtils.FRLogError("singleForm exception ->"+e.getMessage() + addOrUpdate.toString());
            }
        }

        if(delete != null ){
            try{
                accessor.runDMLAction(new DBAction<Boolean>() {
                    @Override
                    public Boolean run(DAOContext context) throws Exception {
                        context.getDAO(FormDefDao.class).remove(delete.getId());
                        return true;
                    }
                });
            }catch(Throwable e){
                FRUtils.FRLogError("singleForm delete exception ->"+e.getMessage() + delete.toString());
            }
        }

        return true;
    }

    /**
     * 批量操作表单字段
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean batch(List<FormDetailEntity> addOrUpdate, List<FormDetailEntity> delete){
        //新增或者删除
        if(addOrUpdate != null && addOrUpdate.size() > 0){
            for(final FormDetailEntity dbe : addOrUpdate){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            FormDetailEntity ae =context.getDAO(FormFieldDao.class).getById(dbe.getId());
                            if(ae != null ){
                                context.getDAO(FormFieldDao.class).update(dbe);
                            }else{
                                context.getDAO(FormFieldDao.class).add(dbe);
                            }
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch addOrUpdate formfield exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        if(delete != null && delete.size() > 0){
            for(final FormDetailEntity dbe : delete){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            context.getDAO(FormFieldDao.class).remove(dbe.getId());
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch  formfiled delete exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        return true;
    }
}
