package com.eco.plugin.wink.workflow.db.controller;

import com.eco.plugin.wink.workflow.db.bean.DBLinkEntity;
import com.eco.plugin.wink.workflow.db.bean.WorkflowProcessEntity;
import com.eco.plugin.wink.workflow.db.dao.DBLinkDao;
import com.eco.plugin.wink.workflow.db.dao.WorkflowProcessDao;
import com.eco.plugin.wink.workflow.utils.FRUtils;
import com.fr.db.fun.impl.AbstractDBAccessProvider;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.db.dao.DAOProvider;

import java.util.ArrayList;
import java.util.List;

/**
 * @author wink
 * @version 10.0
 * Created by wink on 2021-07-29
 **/
@EnableMetrics
public class WorkFlowProcessController extends AbstractDBAccessProvider {

    private static DBAccessor accessor;

    public static DBAccessor getAccessor() {
        return accessor;
    }

    @Override
    public DAOProvider[] registerDAO() {
        return new DAOProvider[]{
                WorkflowProcessDao.DAO
        };
    }

    @Override
    public void onDBAvailable(DBAccessor accessor) {
        WorkFlowProcessController.accessor = accessor;
    }

    /**
     * 批量发起审核
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean batch(List<WorkflowProcessEntity> addOrUpdate, List<WorkflowProcessEntity> delete){
        //新增或者删除
        if(addOrUpdate != null && addOrUpdate.size() > 0){
            for(final WorkflowProcessEntity dbe : addOrUpdate){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            WorkflowProcessEntity ae =context.getDAO(WorkflowProcessDao.class).getById(dbe.getId());
                            if(ae != null ){
                                context.getDAO(WorkflowProcessDao.class).update(dbe);
                            }else{
                                context.getDAO(WorkflowProcessDao.class).add(dbe);
                            }
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch addOrUpdate formfield exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        if(delete != null && delete.size() > 0){
            for(final WorkflowProcessEntity dbe : delete){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            context.getDAO(WorkflowProcessDao.class).remove(dbe.getId());
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch  formfiled delete exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        return true;
    }

    /**
     * 根据formid和dataid获取流程
     * @param
     * @return
     */
    public static List<WorkflowProcessEntity> getByFormidAndDataid(String formid,String dataid){
        try{
            return accessor.runQueryAction(new DBAction<List<WorkflowProcessEntity>>() {
                @Override
                public List<WorkflowProcessEntity> run(DAOContext context) throws Exception {
                    List<WorkflowProcessEntity> result = context.getDAO(WorkflowProcessDao.class).getByFormidAndDataid(formid,dataid);

                    return result == null ? new ArrayList<WorkflowProcessEntity>() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getAllDBLink:"+e.getMessage());
        }

        return new ArrayList<WorkflowProcessEntity>();
    }
}
