package com.eco.plugin.wink.workflow.db.controller;

import com.eco.plugin.wink.workflow.db.bean.WorkflowDefEntity;
import com.eco.plugin.wink.workflow.db.bean.WorkflowNodeEntity;
import com.eco.plugin.wink.workflow.db.dao.WorkflowDefDao;
import com.eco.plugin.wink.workflow.db.dao.WorkflowNodeDao;
import com.eco.plugin.wink.workflow.db.dao.WorkflowDefDao;
import com.eco.plugin.wink.workflow.db.dao.WorkflowNodeDao;
import com.eco.plugin.wink.workflow.utils.FRUtils;
import com.fr.db.fun.impl.AbstractDBAccessProvider;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.db.dao.DAOProvider;

import java.util.ArrayList;
import java.util.List;

/**
 * @author wink
 * @version 10.0
 * Created by wink on 2021-07-29
 **/
@EnableMetrics
public class WorkflowConfigDbController extends AbstractDBAccessProvider {

    private static DBAccessor accessor;

    public static DBAccessor getAccessor() {
        return accessor;
    }

    @Override
    public DAOProvider[] registerDAO() {
        return new DAOProvider[]{
                WorkflowDefDao.DAO, WorkflowNodeDao.DAO
        };
    }

    @Override
    public void onDBAvailable(DBAccessor accessor) {
        WorkflowConfigDbController.accessor = accessor;
    }

    /**
     * 单个操作流程定义
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean singleWorkflowDef(WorkflowDefEntity addOrUpdate, WorkflowDefEntity delete){
        //新增或者删除
        if(addOrUpdate != null ){
            try{
                accessor.runDMLAction(new DBAction<Boolean>() {
                    @Override
                    public Boolean run(DAOContext context) throws Exception {
                        WorkflowDefEntity ae =context.getDAO(WorkflowDefDao.class).getById(addOrUpdate.getId());
                        if(ae != null ){
                            context.getDAO(WorkflowDefDao.class).update(addOrUpdate);
                        }else{
                            context.getDAO(WorkflowDefDao.class).add(addOrUpdate);
                        }
                        return true;
                    }
                });
            }catch(Throwable e){
                FRUtils.FRLogError("singleWorkflowDef exception ->"+e.getMessage() + addOrUpdate.toString());
            }
        }

        if(delete != null ){
            try{
                accessor.runDMLAction(new DBAction<Boolean>() {
                    @Override
                    public Boolean run(DAOContext context) throws Exception {
                        context.getDAO(WorkflowDefDao.class).remove(delete.getId());
                        return true;
                    }
                });
            }catch(Throwable e){
                FRUtils.FRLogError("singleWorkflowDef delete exception ->"+e.getMessage() + delete.toString());
            }
        }

        return true;
    }

    /**
     * 批量操作流程节点
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean batchWorkflowNode(List<WorkflowNodeEntity> addOrUpdate, List<WorkflowNodeEntity> delete){
        //新增或者删除
        if(addOrUpdate != null && addOrUpdate.size() > 0){
            for(final WorkflowNodeEntity dbe : addOrUpdate){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            WorkflowNodeEntity ae =context.getDAO(WorkflowNodeDao.class).getById(dbe.getId());
                            if(ae != null ){
                                context.getDAO(WorkflowNodeDao.class).update(dbe);
                            }else{
                                context.getDAO(WorkflowNodeDao.class).add(dbe);
                            }
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch addOrUpdate batchWorkflowNode exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        if(delete != null && delete.size() > 0){
            for(final WorkflowNodeEntity dbe : delete){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            context.getDAO(WorkflowNodeDao.class).remove(dbe.getId());
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch  batchWorkflowNode delete exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        return true;
    }

    /**
     * 获取所有流程定义信息
     * @param
     * @return
     */
    public static List<WorkflowDefEntity> getWrokflowDefList(){
        try{
            return accessor.runQueryAction(new DBAction<List<WorkflowDefEntity>>() {
                @Override
                public List<WorkflowDefEntity> run(DAOContext context) throws Exception {
                    List<WorkflowDefEntity> result = context.getDAO(WorkflowDefDao.class).getAll();

                    return result == null ? new ArrayList<WorkflowDefEntity>() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getWrokflowDefList:"+e.getMessage());
        }

        return new ArrayList<WorkflowDefEntity>();
    }

    /**
     * 根据表单获取流程定义
     * @param formid
     * @return
     */
    public static WorkflowDefEntity getByFormid(String formid){
        try{
            return accessor.runQueryAction(new DBAction<WorkflowDefEntity>() {
                @Override
                public WorkflowDefEntity run(DAOContext context) throws Exception {
                    WorkflowDefEntity result = context.getDAO(WorkflowDefDao.class).getByFormid(formid);

                    return result == null ? new WorkflowDefEntity() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getWrokflowDefList:"+e.getMessage());
        }

        return new WorkflowDefEntity();
    }

    /**
     * 根据workflowid获取流程节点
     * @param
     * @return
     */
    public static List<WorkflowNodeEntity> getNodesByWorkflowId(String formid){
        try{
            return accessor.runQueryAction(new DBAction<List<WorkflowNodeEntity>>() {
                @Override
                public List<WorkflowNodeEntity> run(DAOContext context) throws Exception {
                    List<WorkflowNodeEntity> result = context.getDAO(WorkflowNodeDao.class).getFieldById(formid);

                    return result == null ? new ArrayList<WorkflowNodeEntity>() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getNodesByWorkflowId:"+e.getMessage());
        }

        return new ArrayList<WorkflowNodeEntity>();
    }
}
