package com.eco.plugin.wink.workflow.kit;

import com.eco.plugin.wink.workflow.utils.FRUserUtils;
import com.eco.plugin.wink.workflow.utils.FRUtils;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.base.constant.SoftRoleType;
import com.fr.decision.authority.base.constant.type.operation.ManualOperationType;
import com.fr.decision.authority.data.CustomRole;
import com.fr.decision.authority.data.User;
import com.fr.decision.base.util.UUIDUtil;
import com.fr.decision.privilege.encrpt.PasswordValidator;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.exception.general.DuplicatedNameException;
import com.fr.decision.webservice.exception.general.SpecialCharProhibitException;
import com.fr.decision.webservice.exception.login.UserPasswordCanNotEmptyException;
import com.fr.decision.webservice.utils.CharLimitType;
import com.fr.decision.webservice.utils.ControllerFactory;
import com.fr.decision.webservice.utils.UserSourceFactory;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.stable.StringUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.data.DataList;
import com.fr.stable.query.restriction.Restriction;
import com.fr.stable.query.restriction.RestrictionFactory;

import java.util.List;

public class UserServiceKit extends UserService {
    private static volatile UserServiceKit userServiceKit = null;

    public UserServiceKit() {
    }

    public static UserServiceKit getInstance() {
        if (userServiceKit == null) {
            userServiceKit = new UserServiceKit();
        }
        return userServiceKit;
    }

    public void editUser(UserBean userBean, String var2) throws Exception {
        this.editUserInfo(userBean);
        if (userBean.isResetPassword()) {
            this.resetPassword(userBean);
        }

        this.updateUserRoles(var2, userBean);
        this.updateUserDepartmentPost(var2, userBean);
    }

    /**
     * 添加用户
     * @param userBean
     * @throws Exception
     */
    public void addUser(UserBean userBean) throws Exception {
        if (StringUtils.isEmpty(userBean.getPassword())) {
            throw new UserPasswordCanNotEmptyException();
        } else {
            String username = userBean.getUsername();
            String mobile = userBean.getMobile();
            String email = userBean.getEmail();
            String id = userBean.getId();
            String psd = userBean.getPassword();
            this.checkUsernameLegal(username, CharLimitType.USER_LIMIT);
            this.checkNonRequiredField(mobile, CharLimitType.MOBILE_LIMIT);
            this.checkNonRequiredField(email, CharLimitType.EMAIL_LIMIT);
            this.checkDuplicatedUser(username);
            String salt = UUIDUtil.generate();
            PasswordValidator var8 = UserSourceFactory.getInstance().getUserSource(ManualOperationType.KEY).getPasswordValidator();
            User user = (new User()).userName(username).realName(userBean.getRealName()).password(var8.encode(username, psd, salt)).salt(salt).email(userBean.getEmail()).mobile(userBean.getMobile()).creationType(ManualOperationType.KEY).lastOperationType(ManualOperationType.KEY).enable(true).id(id);
            FRUtils.FRLogInfo(user.toString());
            AuthorityContext.getInstance().getUserController().add(user);
            this.deleteSoftData(user.getUserName());
        }
    }

    /**
     * 获取角色下所有用户
     * @param rolename
     * @return
     * @throws Exception
     */
    public List<User> getAllUsersByRolename(String rolename) throws Exception{
        UserBean userBean = FRUserUtils.getAdminUser();
        CustomRole customRole = CustomRoleServiceKit.getInstance().getByRolename(rolename);
        return ControllerFactory.getInstance().getUserController(userBean.getUsername()).findAllUsersByCustomRole(userBean.getId(), customRole.getId(), "", false).getList();
    }

    private void checkUsernameLegal(String var1, CharLimitType var2) throws SpecialCharProhibitException {
        if (WebServiceUtils.containSQLChars(var1) || WebServiceUtils.containIllegalChars(var2, var1)) {
            throw new SpecialCharProhibitException();
        }
    }

    private void checkNonRequiredField(String var1, CharLimitType var2) {
        if (StringUtils.isNotEmpty(var1) && WebServiceUtils.containIllegalChars(var2, var1)) {
            throw new SpecialCharProhibitException();
        }
    }

    private void checkDuplicatedUser(String var1) throws Exception {
        User var2 = this.getUserByUserName(var1);
        if (var2 != null) {
            throw new DuplicatedNameException("Duplicated names! ", var1);
        }
    }

    private void deleteSoftData(String var1) throws Exception {
        QueryCondition var2 = QueryFactory.create().addRestriction(RestrictionFactory.and(new Restriction[]{RestrictionFactory.eq("deletedName", var1), RestrictionFactory.eq("type", SoftRoleType.USER)}));
        AuthorityContext.getInstance().getSoftDataController().remove(var2);
    }
}
