package com.eco.plugin.wink.workflow.utils;

import com.eco.plugin.wink.workflow.db.bean.DBLinkEntity;
import com.eco.plugin.wink.workflow.db.controller.WorkFlowController;
import org.apache.commons.dbcp2.BasicDataSourceFactory;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;

public class JDBCUtils {
    private static Properties properties = new Properties();
    private static DataSource dataSource;

    static{
        DBLinkEntity dbLink = WorkFlowController.getAllDBLink().get(0);
        properties.setProperty("driverClassName","com.mysql.jdbc.Driver");
        properties.setProperty("url",dbLink.getUrl()+"?useSSL=false&useUnicode=true&characterEncoding=UTF-8");
        properties.setProperty("username",dbLink.getUsername());
        properties.setProperty("password",dbLink.getPassword());
        properties.setProperty("initialSize","30");
        properties.setProperty("maxTotal","30");
        properties.setProperty("maxIdle","10");
        properties.setProperty("minIdle","5");
        properties.setProperty("maxWailMillis","1000");
        properties.setProperty("removeAbandonedOnMaintenance","true");
        properties.setProperty("removeAbandonedOnBorrow","true");
        properties.setProperty("removeAbandonedTimeout","300");
        try {
            dataSource = BasicDataSourceFactory.createDataSource(properties);
        } catch (Exception e) {
            FRUtils.FRLogError("创建链接池异常:"+e.getMessage());
        }


    }

    public static Connection getConnect(){
        Connection connection = null;

        try{
            connection = dataSource.getConnection();
        }catch(Exception e){
            FRUtils.FRLogError("获取数据库连接异常:"+e.getMessage());
        }

        return connection;
    }

    /**
     * 更新
     * @param sql
     * @return
     */
    public static int update(String sql){
        Connection conn=null;
        Statement st=null;
        int count = 0;

        try {
            conn = getConnect();
            st = conn.createStatement();
            count = st.executeUpdate(sql);

        } catch (SQLException e) {
            FRUtils.FRLogError("执行updata语句异常");
        } finally {
            release(conn, st, null);
        }

        return count;
    }

    /**
     * 获取数据
     * @param sql
     * @param columns
     * @return
     */
    public static List<Map<String,Object>> getResult(String sql, String[] columns,String formid){
        List<Map<String,Object>> result = new ArrayList<Map<String,Object>>();

        Connection conn=null;
        Statement st=null;
        ResultSet rs=null;

        try {
            conn = getConnect();
            st = conn.createStatement();
            rs = st.executeQuery(sql);
            //遍历查询每条记录
            while(rs.next()) {
                Map<String,Object> map = new HashMap<String,Object>();

                for(String column : columns){
                    map.put(column,rs.getString(column));
                }
                map.put("formid",formid);
                result.add(map);
            }
        } catch (SQLException e) {
            FRUtils.FRLogInfo("获取数据异常:"+e.getMessage());
        } finally {
            release(conn, st, rs);
        }

        return result;
    }

    /**
     * 获取数量
     * @param sql
     * @return
     */
    public static int getCount(String sql){
        int count = 0;

        Connection conn=null;
        Statement st=null;
        ResultSet rs=null;

        try {
            conn = getConnect();
            st = conn.createStatement();
            rs = st.executeQuery(sql);
            //遍历查询每条记录
            while(rs.next()) {
                count = rs.getInt(1);
            }
        } catch (SQLException e) {
            FRUtils.FRLogInfo("获取数量异常:"+e.getMessage());
        } finally {
            release(conn, st, rs);
        }

        return count;
    }

    private static void release(Connection conn, Statement st, ResultSet rs) {
        closeRs(rs);
        closeSt(st);
        closeConn(conn);
    }

    private static void closeRs(ResultSet rs) {
        try {
            if(rs!=null) {
                rs.close();
            }
        } catch (SQLException e) {
            FRUtils.FRLogError("关闭rs异常");
        } finally {
            rs=null;
        }

    }

    private static void closeSt(Statement st) {
        try {
            if(st!=null) {
                st.close();
            }
        } catch (SQLException e) {
            FRUtils.FRLogError("关闭st异常");
        } finally {
            st=null;
        }

    }

    private static void closeConn(Connection conn) {
        try {
            if(conn!=null) {
                conn.close();
            }
        } catch (SQLException e) {
            FRUtils.FRLogError("关闭connect异常");
        } finally {
            conn=null;
        }
    }

}
